# This file was auto-generated by Fern from our API Definition.

import typing

TransactionLedgerType = typing.Union[
    typing.Literal[
        "ASSET",
        "LIABILITY",
        "EQUITY",
        "REVENUE",
        "EXPENSE",
        "INTEREST_INCOME",
        "COGS_EXPENSE",
        "SGA_EXPENSE",
        "SM_EXPENSE",
        "RD_EXPENSE",
        "INTEREST_EXPENSE",
        "TAX_EXPENSE",
        "COMMODITY",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...types.transaction import Transaction
import typing_extensions
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class GetTransactionsByMonthResponse(UniversalBaseModel):
    transactions: typing.Optional[typing.List[Transaction]] = None
    total_count: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="totalCount")] = pydantic.Field(
        default=None
    )
    """
    Total number of transactions
    """

    month: typing.Optional[str] = pydantic.Field(default=None)
    """
    Month in YYYY-MM format
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ExportTransactionRequestFormat,
    ExportTransactionResponse,
    GetIdTransactionsResponse,
    GetTransactionsByMonthResponse,
    SuggestTransactionCategoriesResponseItem,
)

__all__ = [
    "ExportTransactionRequestFormat",
    "ExportTransactionResponse",
    "GetIdTransactionsResponse",
    "GetTransactionsByMonthResponse",
    "SuggestTransactionCategoriesResponseItem",
]



# This file was auto-generated by Fern from our API Definition.

from .category import Category
from .category_type import CategoryType
from .company import Company
from .company_entity_type import CompanyEntityType
from .company_status import CompanyStatus
from .developer import Developer
from .error import Error
from .plaid_account import PlaidAccount
from .plaid_account_status import PlaidAccountStatus
from .preferences import Preferences
from .transaction import Transaction
from .transaction_categorization_status import TransactionCategorizationStatus
from .transaction_direction import TransactionDirection
from .transaction_ledger_type import TransactionLedgerType
from .transaction_status import TransactionStatus
from .transaction_transaction_type import TransactionTransactionType

__all__ = [
    "Category",
    "CategoryType",
    "Company",
    "CompanyEntityType",
    "CompanyStatus",
    "Developer",
    "Error",
    "PlaidAccount",
    "PlaidAccountStatus",
    "Preferences",
    "Transaction",
    "TransactionCategorizationStatus",
    "TransactionDirection",
    "TransactionLedgerType",
    "TransactionStatus",
    "TransactionTransactionType",
]



# This file was auto-generated by Fern from our API Definition.

import typing

TransactionCategorizationStatus = typing.Union[typing.Literal["PENDING", "READY_FOR_INPUT", "CATEGORIZED"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .category_type import CategoryType
import typing_extensions
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Category(UniversalBaseModel):
    id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Unique identifier for the category
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the category
    """

    type: typing.Optional[CategoryType] = pydantic.Field(default=None)
    """
    Type of the category
    """

    company_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="companyId")] = pydantic.Field(
        default=None
    )
    """
    ID of the company
    """

    account_code: typing.Optional[int] = pydantic.Field(default=None)
    """
    Account code
    """

    sub_type_code: typing.Optional[int] = pydantic.Field(default=None)
    """
    Sub-type code
    """

    tb_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Trial balance ID
    """

    ledger_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Ledger ID
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Category,
    CategoryType,
    Company,
    CompanyEntityType,
    CompanyStatus,
    Developer,
    Error,
    PlaidAccount,
    PlaidAccountStatus,
    Preferences,
    Transaction,
    TransactionCategorizationStatus,
    TransactionDirection,
    TransactionLedgerType,
    TransactionStatus,
    TransactionTransactionType,
)
from .errors import BadRequestError, NotFoundError
from . import categories, companies, transactions
from .categories import GetCategoriesByCompanyResponseItem
from .client import AsyncOpenLedgerClient, OpenLedgerClient
from .environment import OpenLedgerClientEnvironment
from .transactions import (
    ExportTransactionRequestFormat,
    ExportTransactionResponse,
    GetIdTransactionsResponse,
    GetTransactionsByMonthResponse,
    SuggestTransactionCategoriesResponseItem,
)
from .version import __version__

__all__ = [
    "AsyncOpenLedgerClient",
    "BadRequestError",
    "Category",
    "CategoryType",
    "Company",
    "CompanyEntityType",
    "CompanyStatus",
    "Developer",
    "Error",
    "ExportTransactionRequestFormat",
    "ExportTransactionResponse",
    "GetCategoriesByCompanyResponseItem",
    "GetIdTransactionsResponse",
    "GetTransactionsByMonthResponse",
    "NotFoundError",
    "OpenLedgerClient",
    "OpenLedgerClientEnvironment",
    "PlaidAccount",
    "PlaidAccountStatus",
    "Preferences",
    "SuggestTransactionCategoriesResponseItem",
    "Transaction",
    "TransactionCategorizationStatus",
    "TransactionDirection",
    "TransactionLedgerType",
    "TransactionStatus",
    "TransactionTransactionType",
    "__version__",
    "categories",
    "companies",
    "transactions",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from .types.get_id_transactions_response import GetIdTransactionsResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..errors.not_found_error import NotFoundError
from ..types.error import Error
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
import datetime as dt
from ..types.transaction_status import TransactionStatus
from ..types.transaction_transaction_type import TransactionTransactionType
from ..types.transaction_ledger_type import TransactionLedgerType
from ..types.transaction_direction import TransactionDirection
from ..types.transaction_categorization_status import TransactionCategorizationStatus
from ..types.transaction import Transaction
from ..errors.bad_request_error import BadRequestError
from .types.export_transaction_request_format import ExportTransactionRequestFormat
from .types.export_transaction_response import ExportTransactionResponse
from .types.get_transactions_by_month_response import GetTransactionsByMonthResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from .types.suggest_transaction_categories_response_item import SuggestTransactionCategoriesResponseItem
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TransactionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_transactions_by_company(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetIdTransactionsResponse:
        """
        Parameters
        ----------
        id : str
            Company ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetIdTransactionsResponse
            List of transactions

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.transactions.get_transactions_by_company(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetIdTransactionsResponse,
                    parse_obj_as(
                        type_=GetIdTransactionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_a_new_transaction(
        self,
        id_: str,
        *,
        id: typing.Optional[str] = OMIT,
        date: typing.Optional[dt.datetime] = OMIT,
        amount: typing.Optional[float] = OMIT,
        currency: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        status: typing.Optional[TransactionStatus] = OMIT,
        created_by: typing.Optional[int] = OMIT,
        updated_by: typing.Optional[int] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        bank_transaction_id: typing.Optional[str] = OMIT,
        transaction_type: typing.Optional[TransactionTransactionType] = OMIT,
        ledger_type: typing.Optional[TransactionLedgerType] = OMIT,
        bank_account_id: typing.Optional[str] = OMIT,
        business_id: typing.Optional[str] = OMIT,
        direction: typing.Optional[TransactionDirection] = OMIT,
        balance: typing.Optional[float] = OMIT,
        counterparty_name: typing.Optional[str] = OMIT,
        categorization_status: typing.Optional[TransactionCategorizationStatus] = OMIT,
        category_id: typing.Optional[int] = OMIT,
        company_id: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Transaction:
        """
        Parameters
        ----------
        id_ : str
            Company ID

        id : typing.Optional[str]
            Unique identifier for the transaction

        date : typing.Optional[dt.datetime]
            Date of the transaction

        amount : typing.Optional[float]
            Amount of the transaction

        currency : typing.Optional[str]
            Currency of the transaction

        description : typing.Optional[str]
            Description of the transaction

        status : typing.Optional[TransactionStatus]
            Status of the transaction

        created_by : typing.Optional[int]
            ID of the user who created the transaction

        updated_by : typing.Optional[int]
            ID of the user who last updated the transaction

        created_at : typing.Optional[dt.datetime]
            Date when the transaction was created

        updated_at : typing.Optional[dt.datetime]
            Date when the transaction was last updated

        bank_transaction_id : typing.Optional[str]
            ID of the bank transaction

        transaction_type : typing.Optional[TransactionTransactionType]
            Type of transaction

        ledger_type : typing.Optional[TransactionLedgerType]
            Ledger type of the transaction

        bank_account_id : typing.Optional[str]
            ID of the bank account

        business_id : typing.Optional[str]
            ID of the business

        direction : typing.Optional[TransactionDirection]
            Direction of the transaction

        balance : typing.Optional[float]
            Balance after the transaction

        counterparty_name : typing.Optional[str]
            Name of the counterparty

        categorization_status : typing.Optional[TransactionCategorizationStatus]
            Status of categorization

        category_id : typing.Optional[int]
            ID of the category

        company_id : typing.Optional[str]
            ID of the company

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Additional metadata for the transaction

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Transaction
            Transaction created successfully

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.transactions.create_a_new_transaction(
            id_="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id_)}/transactions",
            method="POST",
            json={
                "id": id,
                "date": date,
                "amount": amount,
                "currency": currency,
                "description": description,
                "status": status,
                "created_by": created_by,
                "updated_by": updated_by,
                "created_at": created_at,
                "updated_at": updated_at,
                "bank_transaction_id": bank_transaction_id,
                "transaction_type": transaction_type,
                "ledger_type": ledger_type,
                "bank_account_id": bank_account_id,
                "business_id": business_id,
                "direction": direction,
                "balance": balance,
                "counterparty_name": counterparty_name,
                "categorizationStatus": categorization_status,
                "category_id": category_id,
                "company_id": company_id,
                "metadata": metadata,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Transaction,
                    parse_obj_as(
                        type_=Transaction,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def edit_a_transaction(
        self,
        id_: str,
        transaction_id: str,
        *,
        id: typing.Optional[str] = OMIT,
        date: typing.Optional[dt.datetime] = OMIT,
        amount: typing.Optional[float] = OMIT,
        currency: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        status: typing.Optional[TransactionStatus] = OMIT,
        created_by: typing.Optional[int] = OMIT,
        updated_by: typing.Optional[int] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        bank_transaction_id: typing.Optional[str] = OMIT,
        transaction_type: typing.Optional[TransactionTransactionType] = OMIT,
        ledger_type: typing.Optional[TransactionLedgerType] = OMIT,
        bank_account_id: typing.Optional[str] = OMIT,
        business_id: typing.Optional[str] = OMIT,
        direction: typing.Optional[TransactionDirection] = OMIT,
        balance: typing.Optional[float] = OMIT,
        counterparty_name: typing.Optional[str] = OMIT,
        categorization_status: typing.Optional[TransactionCategorizationStatus] = OMIT,
        category_id: typing.Optional[int] = OMIT,
        company_id: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Transaction:
        """
        Parameters
        ----------
        id_ : str
            Company ID

        transaction_id : str
            Transaction ID

        id : typing.Optional[str]
            Unique identifier for the transaction

        date : typing.Optional[dt.datetime]
            Date of the transaction

        amount : typing.Optional[float]
            Amount of the transaction

        currency : typing.Optional[str]
            Currency of the transaction

        description : typing.Optional[str]
            Description of the transaction

        status : typing.Optional[TransactionStatus]
            Status of the transaction

        created_by : typing.Optional[int]
            ID of the user who created the transaction

        updated_by : typing.Optional[int]
            ID of the user who last updated the transaction

        created_at : typing.Optional[dt.datetime]
            Date when the transaction was created

        updated_at : typing.Optional[dt.datetime]
            Date when the transaction was last updated

        bank_transaction_id : typing.Optional[str]
            ID of the bank transaction

        transaction_type : typing.Optional[TransactionTransactionType]
            Type of transaction

        ledger_type : typing.Optional[TransactionLedgerType]
            Ledger type of the transaction

        bank_account_id : typing.Optional[str]
            ID of the bank account

        business_id : typing.Optional[str]
            ID of the business

        direction : typing.Optional[TransactionDirection]
            Direction of the transaction

        balance : typing.Optional[float]
            Balance after the transaction

        counterparty_name : typing.Optional[str]
            Name of the counterparty

        categorization_status : typing.Optional[TransactionCategorizationStatus]
            Status of categorization

        category_id : typing.Optional[int]
            ID of the category

        company_id : typing.Optional[str]
            ID of the company

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Additional metadata for the transaction

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Transaction
            Transaction updated successfully

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.transactions.edit_a_transaction(
            id_="id",
            transaction_id="transactionId",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id_)}/transactions/{jsonable_encoder(transaction_id)}",
            method="PUT",
            json={
                "id": id,
                "date": date,
                "amount": amount,
                "currency": currency,
                "description": description,
                "status": status,
                "created_by": created_by,
                "updated_by": updated_by,
                "created_at": created_at,
                "updated_at": updated_at,
                "bank_transaction_id": bank_transaction_id,
                "transaction_type": transaction_type,
                "ledger_type": ledger_type,
                "bank_account_id": bank_account_id,
                "business_id": business_id,
                "direction": direction,
                "balance": balance,
                "counterparty_name": counterparty_name,
                "categorizationStatus": categorization_status,
                "category_id": category_id,
                "company_id": company_id,
                "metadata": metadata,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Transaction,
                    parse_obj_as(
                        type_=Transaction,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def export_transaction(
        self,
        id: str,
        *,
        format: typing.Optional[ExportTransactionRequestFormat] = None,
        start_date: typing.Optional[str] = None,
        end_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExportTransactionResponse:
        """
        Parameters
        ----------
        id : str
            Company ID

        format : typing.Optional[ExportTransactionRequestFormat]
            Export format

        start_date : typing.Optional[str]
            Start date for filtering transactions

        end_date : typing.Optional[str]
            End date for filtering transactions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExportTransactionResponse
            Transactions exported successfully

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.transactions.export_transaction(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/export",
            method="GET",
            params={
                "format": format,
                "startDate": start_date,
                "endDate": end_date,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ExportTransactionResponse,
                    parse_obj_as(
                        type_=ExportTransactionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_transactions_by_month(
        self, id: str, month: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTransactionsByMonthResponse:
        """
        Parameters
        ----------
        id : str
            Company ID

        month : str
            Month in YYYY-MM format

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTransactionsByMonthResponse
            List of transactions for the specified month

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.transactions.get_transactions_by_month(
            id="id",
            month="month",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/month/{jsonable_encoder(month)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetTransactionsByMonthResponse,
                    parse_obj_as(
                        type_=GetTransactionsByMonthResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def prompt_transaction(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        id : str
            Company ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Transaction prompt successful

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.transactions.prompt_transaction(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/prompt",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def classify_transaction(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Transaction]:
        """
        Parameters
        ----------
        id : str
            Company ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Transaction]
            Transactions classified successfully

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.transactions.classify_transaction(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/classify",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Transaction],
                    parse_obj_as(
                        type_=typing.List[Transaction],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def generate_general_ledger(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        id : str
            Company ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            General ledger generated successfully

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.transactions.generate_general_ledger(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/general-ledger",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def bulk_create_transactions(
        self, id: str, *, request: typing.Sequence[Transaction], request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Transaction]:
        """
        Parameters
        ----------
        id : str
            Company ID

        request : typing.Sequence[Transaction]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Transaction]
            Transactions created successfully

        Examples
        --------
        from openledger import OpenLedgerClient, Transaction

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.transactions.bulk_create_transactions(
            id="id",
            request=[Transaction()],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/bulk",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=typing.Sequence[Transaction], direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Transaction],
                    parse_obj_as(
                        type_=typing.List[Transaction],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def suggest_transaction_categories(
        self,
        id: str,
        *,
        transactions: typing.Optional[typing.Sequence[Transaction]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SuggestTransactionCategoriesResponseItem]:
        """
        Parameters
        ----------
        id : str
            Company ID

        transactions : typing.Optional[typing.Sequence[Transaction]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SuggestTransactionCategoriesResponseItem]
            Categories suggested successfully

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.transactions.suggest_transaction_categories(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/suggest-categories",
            method="POST",
            json={
                "transactions": convert_and_respect_annotation_metadata(
                    object_=transactions, annotation=typing.Sequence[Transaction], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[SuggestTransactionCategoriesResponseItem],
                    parse_obj_as(
                        type_=typing.List[SuggestTransactionCategoriesResponseItem],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTransactionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_transactions_by_company(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetIdTransactionsResponse:
        """
        Parameters
        ----------
        id : str
            Company ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetIdTransactionsResponse
            List of transactions

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transactions.get_transactions_by_company(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetIdTransactionsResponse,
                    parse_obj_as(
                        type_=GetIdTransactionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_a_new_transaction(
        self,
        id_: str,
        *,
        id: typing.Optional[str] = OMIT,
        date: typing.Optional[dt.datetime] = OMIT,
        amount: typing.Optional[float] = OMIT,
        currency: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        status: typing.Optional[TransactionStatus] = OMIT,
        created_by: typing.Optional[int] = OMIT,
        updated_by: typing.Optional[int] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        bank_transaction_id: typing.Optional[str] = OMIT,
        transaction_type: typing.Optional[TransactionTransactionType] = OMIT,
        ledger_type: typing.Optional[TransactionLedgerType] = OMIT,
        bank_account_id: typing.Optional[str] = OMIT,
        business_id: typing.Optional[str] = OMIT,
        direction: typing.Optional[TransactionDirection] = OMIT,
        balance: typing.Optional[float] = OMIT,
        counterparty_name: typing.Optional[str] = OMIT,
        categorization_status: typing.Optional[TransactionCategorizationStatus] = OMIT,
        category_id: typing.Optional[int] = OMIT,
        company_id: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Transaction:
        """
        Parameters
        ----------
        id_ : str
            Company ID

        id : typing.Optional[str]
            Unique identifier for the transaction

        date : typing.Optional[dt.datetime]
            Date of the transaction

        amount : typing.Optional[float]
            Amount of the transaction

        currency : typing.Optional[str]
            Currency of the transaction

        description : typing.Optional[str]
            Description of the transaction

        status : typing.Optional[TransactionStatus]
            Status of the transaction

        created_by : typing.Optional[int]
            ID of the user who created the transaction

        updated_by : typing.Optional[int]
            ID of the user who last updated the transaction

        created_at : typing.Optional[dt.datetime]
            Date when the transaction was created

        updated_at : typing.Optional[dt.datetime]
            Date when the transaction was last updated

        bank_transaction_id : typing.Optional[str]
            ID of the bank transaction

        transaction_type : typing.Optional[TransactionTransactionType]
            Type of transaction

        ledger_type : typing.Optional[TransactionLedgerType]
            Ledger type of the transaction

        bank_account_id : typing.Optional[str]
            ID of the bank account

        business_id : typing.Optional[str]
            ID of the business

        direction : typing.Optional[TransactionDirection]
            Direction of the transaction

        balance : typing.Optional[float]
            Balance after the transaction

        counterparty_name : typing.Optional[str]
            Name of the counterparty

        categorization_status : typing.Optional[TransactionCategorizationStatus]
            Status of categorization

        category_id : typing.Optional[int]
            ID of the category

        company_id : typing.Optional[str]
            ID of the company

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Additional metadata for the transaction

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Transaction
            Transaction created successfully

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transactions.create_a_new_transaction(
                id_="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id_)}/transactions",
            method="POST",
            json={
                "id": id,
                "date": date,
                "amount": amount,
                "currency": currency,
                "description": description,
                "status": status,
                "created_by": created_by,
                "updated_by": updated_by,
                "created_at": created_at,
                "updated_at": updated_at,
                "bank_transaction_id": bank_transaction_id,
                "transaction_type": transaction_type,
                "ledger_type": ledger_type,
                "bank_account_id": bank_account_id,
                "business_id": business_id,
                "direction": direction,
                "balance": balance,
                "counterparty_name": counterparty_name,
                "categorizationStatus": categorization_status,
                "category_id": category_id,
                "company_id": company_id,
                "metadata": metadata,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Transaction,
                    parse_obj_as(
                        type_=Transaction,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def edit_a_transaction(
        self,
        id_: str,
        transaction_id: str,
        *,
        id: typing.Optional[str] = OMIT,
        date: typing.Optional[dt.datetime] = OMIT,
        amount: typing.Optional[float] = OMIT,
        currency: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        status: typing.Optional[TransactionStatus] = OMIT,
        created_by: typing.Optional[int] = OMIT,
        updated_by: typing.Optional[int] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        bank_transaction_id: typing.Optional[str] = OMIT,
        transaction_type: typing.Optional[TransactionTransactionType] = OMIT,
        ledger_type: typing.Optional[TransactionLedgerType] = OMIT,
        bank_account_id: typing.Optional[str] = OMIT,
        business_id: typing.Optional[str] = OMIT,
        direction: typing.Optional[TransactionDirection] = OMIT,
        balance: typing.Optional[float] = OMIT,
        counterparty_name: typing.Optional[str] = OMIT,
        categorization_status: typing.Optional[TransactionCategorizationStatus] = OMIT,
        category_id: typing.Optional[int] = OMIT,
        company_id: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Transaction:
        """
        Parameters
        ----------
        id_ : str
            Company ID

        transaction_id : str
            Transaction ID

        id : typing.Optional[str]
            Unique identifier for the transaction

        date : typing.Optional[dt.datetime]
            Date of the transaction

        amount : typing.Optional[float]
            Amount of the transaction

        currency : typing.Optional[str]
            Currency of the transaction

        description : typing.Optional[str]
            Description of the transaction

        status : typing.Optional[TransactionStatus]
            Status of the transaction

        created_by : typing.Optional[int]
            ID of the user who created the transaction

        updated_by : typing.Optional[int]
            ID of the user who last updated the transaction

        created_at : typing.Optional[dt.datetime]
            Date when the transaction was created

        updated_at : typing.Optional[dt.datetime]
            Date when the transaction was last updated

        bank_transaction_id : typing.Optional[str]
            ID of the bank transaction

        transaction_type : typing.Optional[TransactionTransactionType]
            Type of transaction

        ledger_type : typing.Optional[TransactionLedgerType]
            Ledger type of the transaction

        bank_account_id : typing.Optional[str]
            ID of the bank account

        business_id : typing.Optional[str]
            ID of the business

        direction : typing.Optional[TransactionDirection]
            Direction of the transaction

        balance : typing.Optional[float]
            Balance after the transaction

        counterparty_name : typing.Optional[str]
            Name of the counterparty

        categorization_status : typing.Optional[TransactionCategorizationStatus]
            Status of categorization

        category_id : typing.Optional[int]
            ID of the category

        company_id : typing.Optional[str]
            ID of the company

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Additional metadata for the transaction

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Transaction
            Transaction updated successfully

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transactions.edit_a_transaction(
                id_="id",
                transaction_id="transactionId",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id_)}/transactions/{jsonable_encoder(transaction_id)}",
            method="PUT",
            json={
                "id": id,
                "date": date,
                "amount": amount,
                "currency": currency,
                "description": description,
                "status": status,
                "created_by": created_by,
                "updated_by": updated_by,
                "created_at": created_at,
                "updated_at": updated_at,
                "bank_transaction_id": bank_transaction_id,
                "transaction_type": transaction_type,
                "ledger_type": ledger_type,
                "bank_account_id": bank_account_id,
                "business_id": business_id,
                "direction": direction,
                "balance": balance,
                "counterparty_name": counterparty_name,
                "categorizationStatus": categorization_status,
                "category_id": category_id,
                "company_id": company_id,
                "metadata": metadata,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Transaction,
                    parse_obj_as(
                        type_=Transaction,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def export_transaction(
        self,
        id: str,
        *,
        format: typing.Optional[ExportTransactionRequestFormat] = None,
        start_date: typing.Optional[str] = None,
        end_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExportTransactionResponse:
        """
        Parameters
        ----------
        id : str
            Company ID

        format : typing.Optional[ExportTransactionRequestFormat]
            Export format

        start_date : typing.Optional[str]
            Start date for filtering transactions

        end_date : typing.Optional[str]
            End date for filtering transactions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExportTransactionResponse
            Transactions exported successfully

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transactions.export_transaction(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/export",
            method="GET",
            params={
                "format": format,
                "startDate": start_date,
                "endDate": end_date,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ExportTransactionResponse,
                    parse_obj_as(
                        type_=ExportTransactionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_transactions_by_month(
        self, id: str, month: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTransactionsByMonthResponse:
        """
        Parameters
        ----------
        id : str
            Company ID

        month : str
            Month in YYYY-MM format

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTransactionsByMonthResponse
            List of transactions for the specified month

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transactions.get_transactions_by_month(
                id="id",
                month="month",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/month/{jsonable_encoder(month)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetTransactionsByMonthResponse,
                    parse_obj_as(
                        type_=GetTransactionsByMonthResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def prompt_transaction(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        id : str
            Company ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Transaction prompt successful

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transactions.prompt_transaction(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/prompt",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def classify_transaction(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Transaction]:
        """
        Parameters
        ----------
        id : str
            Company ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Transaction]
            Transactions classified successfully

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transactions.classify_transaction(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/classify",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Transaction],
                    parse_obj_as(
                        type_=typing.List[Transaction],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def generate_general_ledger(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        id : str
            Company ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            General ledger generated successfully

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transactions.generate_general_ledger(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/general-ledger",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def bulk_create_transactions(
        self, id: str, *, request: typing.Sequence[Transaction], request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Transaction]:
        """
        Parameters
        ----------
        id : str
            Company ID

        request : typing.Sequence[Transaction]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Transaction]
            Transactions created successfully

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient, Transaction

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transactions.bulk_create_transactions(
                id="id",
                request=[Transaction()],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/bulk",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=typing.Sequence[Transaction], direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Transaction],
                    parse_obj_as(
                        type_=typing.List[Transaction],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def suggest_transaction_categories(
        self,
        id: str,
        *,
        transactions: typing.Optional[typing.Sequence[Transaction]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SuggestTransactionCategoriesResponseItem]:
        """
        Parameters
        ----------
        id : str
            Company ID

        transactions : typing.Optional[typing.Sequence[Transaction]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SuggestTransactionCategoriesResponseItem]
            Categories suggested successfully

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transactions.suggest_transaction_categories(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/transactions/suggest-categories",
            method="POST",
            json={
                "transactions": convert_and_respect_annotation_metadata(
                    object_=transactions, annotation=typing.Sequence[Transaction], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[SuggestTransactionCategoriesResponseItem],
                    parse_obj_as(
                        type_=typing.List[SuggestTransactionCategoriesResponseItem],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

TransactionTransactionType = typing.Union[
    typing.Literal["INVOICE", "PAYMENT", "BANK_TRANSACTION", "BANK_TRANSACTION_PLAID"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import datetime as dt
from .plaid_account_status import PlaidAccountStatus
import typing_extensions
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class PlaidAccount(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the Plaid account
    """

    plaid_access_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    Plaid access token
    """

    plaid_item_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Plaid item ID
    """

    plaid_institution_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Plaid institution ID
    """

    plaid_account_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Plaid account ID
    """

    account_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the account
    """

    account_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type of the account
    """

    account_subtype: typing.Optional[str] = pydantic.Field(default=None)
    """
    Subtype of the account
    """

    account_mask: typing.Optional[str] = pydantic.Field(default=None)
    """
    Masked account number
    """

    linked_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Date when the account was linked
    """

    status: typing.Optional[PlaidAccountStatus] = pydantic.Field(default=None)
    """
    Status of the account
    """

    company_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="companyId")] = pydantic.Field(
        default=None
    )
    """
    ID of the company
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cursor for pagination
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import datetime as dt
from .transaction_status import TransactionStatus
from .transaction_transaction_type import TransactionTransactionType
from .transaction_ledger_type import TransactionLedgerType
from .transaction_direction import TransactionDirection
import typing_extensions
from .transaction_categorization_status import TransactionCategorizationStatus
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Transaction(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the transaction
    """

    date: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Date of the transaction
    """

    amount: typing.Optional[float] = pydantic.Field(default=None)
    """
    Amount of the transaction
    """

    currency: typing.Optional[str] = pydantic.Field(default=None)
    """
    Currency of the transaction
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the transaction
    """

    status: typing.Optional[TransactionStatus] = pydantic.Field(default=None)
    """
    Status of the transaction
    """

    created_by: typing.Optional[int] = pydantic.Field(default=None)
    """
    ID of the user who created the transaction
    """

    updated_by: typing.Optional[int] = pydantic.Field(default=None)
    """
    ID of the user who last updated the transaction
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Date when the transaction was created
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Date when the transaction was last updated
    """

    bank_transaction_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the bank transaction
    """

    transaction_type: typing.Optional[TransactionTransactionType] = pydantic.Field(default=None)
    """
    Type of transaction
    """

    ledger_type: typing.Optional[TransactionLedgerType] = pydantic.Field(default=None)
    """
    Ledger type of the transaction
    """

    bank_account_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the bank account
    """

    business_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the business
    """

    direction: typing.Optional[TransactionDirection] = pydantic.Field(default=None)
    """
    Direction of the transaction
    """

    balance: typing.Optional[float] = pydantic.Field(default=None)
    """
    Balance after the transaction
    """

    counterparty_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the counterparty
    """

    categorization_status: typing_extensions.Annotated[
        typing.Optional[TransactionCategorizationStatus], FieldMetadata(alias="categorizationStatus")
    ] = pydantic.Field(default=None)
    """
    Status of categorization
    """

    category_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    ID of the category
    """

    company_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the company
    """

    metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Additional metadata for the transaction
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from .types.get_categories_by_company_response_item import GetCategoriesByCompanyResponseItem
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CategoriesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_categories_by_company(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[GetCategoriesByCompanyResponseItem]:
        """
        Parameters
        ----------
        id : str
            Company ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[GetCategoriesByCompanyResponseItem]
            List of categories

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.categories.get_categories_by_company(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/company/category",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[GetCategoriesByCompanyResponseItem],
                    parse_obj_as(
                        type_=typing.List[GetCategoriesByCompanyResponseItem],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_category(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        id : str
            Company ID

        name : typing.Optional[str]

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Category created successfully

        Examples
        --------
        from openledger import OpenLedgerClient

        client = OpenLedgerClient(
            token="YOUR_TOKEN",
        )
        client.categories.create_category(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/company/category",
            method="PUT",
            json={
                "name": name,
                "description": description,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncCategoriesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_categories_by_company(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[GetCategoriesByCompanyResponseItem]:
        """
        Parameters
        ----------
        id : str
            Company ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[GetCategoriesByCompanyResponseItem]
            List of categories

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.categories.get_categories_by_company(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/company/category",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[GetCategoriesByCompanyResponseItem],
                    parse_obj_as(
                        type_=typing.List[GetCategoriesByCompanyResponseItem],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_category(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        id : str
            Company ID

        name : typing.Optional[str]

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Category created successfully

        Examples
        --------
        import asyncio

        from openledger import AsyncOpenLedgerClient

        client = AsyncOpenLedgerClient(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.categories.create_category(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"{jsonable_encoder(id)}/company/category",
            method="PUT",
            json={
                "name": name,
                "description": description,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

CompanyEntityType = typing.Union[
    typing.Literal["LLC", "CORPORATION", "PARTNERSHIP", "SOLE_PROPRIETORSHIP", "NON_PROFIT"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from .get_categories_by_company_response_item import GetCategoriesByCompanyResponseItem

__all__ = ["GetCategoriesByCompanyResponseItem"]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Developer(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the developer
    """

    name: str = pydantic.Field()
    """
    Name of the developer
    """

    developer_id: str = pydantic.Field()
    """
    External identifier for the developer
    """

    date_created: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Date when the developer was created
    """

    industry: typing.Optional[str] = pydantic.Field(default=None)
    """
    Industry the developer operates in
    """

    preferences_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    ID of the developer's preferences
    """

    api_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    API key for the developer
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .company_entity_type import CompanyEntityType
import datetime as dt
from .company_status import CompanyStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Company(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the company
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    External identifier for the company
    """

    legal_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Legal name of the company
    """

    tin: typing.Optional[str] = pydantic.Field(default=None)
    """
    Tax identification number
    """

    us_state: typing.Optional[str] = pydantic.Field(default=None)
    """
    US state where the company is registered
    """

    entity_type: typing.Optional[CompanyEntityType] = pydantic.Field(default=None)
    """
    Type of business entity
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Contact phone number
    """

    developer_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the developer who created this company
    """

    date_created: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Date when the company was created
    """

    google_auth_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    Google authentication token
    """

    admin_user_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    ID of the admin user
    """

    address_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    ID of the company's address
    """

    phone: typing.Optional[str] = pydantic.Field(default=None)
    """
    Alternative phone number
    """

    website: typing.Optional[str] = pydantic.Field(default=None)
    """
    Company website URL
    """

    logo_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL to company logo
    """

    status: typing.Optional[CompanyStatus] = pydantic.Field(default=None)
    """
    Current status of the company
    """

    subscription_plan: typing.Optional[str] = pydantic.Field(default=None)
    """
    Subscription plan of the company
    """

    billing_information: typing.Optional[str] = pydantic.Field(default=None)
    """
    Billing information for the company
    """

    settings_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    ID of the company settings
    """

    new: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the company is newly created
    """

    ledger: typing.Optional[str] = pydantic.Field(default=None)
    """
    Ledger information
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .export_transaction_request_format import ExportTransactionRequestFormat
from .export_transaction_response import ExportTransactionResponse
from .get_id_transactions_response import GetIdTransactionsResponse
from .get_transactions_by_month_response import GetTransactionsByMonthResponse
from .suggest_transaction_categories_response_item import SuggestTransactionCategoriesResponseItem

__all__ = [
    "ExportTransactionRequestFormat",
    "ExportTransactionResponse",
    "GetIdTransactionsResponse",
    "GetTransactionsByMonthResponse",
    "SuggestTransactionCategoriesResponseItem",
]
